#include "precomp.h"

CSiUtilEx::CSiUtilEx(HRESULT* phr)
    : m_pTypeInfo(nullptr), m_lCommPort(1), m_nDisableDialogBoxes(0), m_lRefCount(1)
{
    ::LockServer();

    ITypeLib* pTlb = nullptr;
    if(SUCCEEDED(*phr = LoadRegTypeLib(LIBID_SiUtilExLib, 1, 0, LOCALE_USER_DEFAULT, &pTlb)))
    {
        *phr = pTlb->GetTypeInfoOfGuid(IID_ISiUtilEx, &m_pTypeInfo);
        pTlb->Release();
    }
}

CSiUtilEx::~CSiUtilEx()
{
    if(m_pTypeInfo)
        m_pTypeInfo->Release();

    ::UnlockServer();
}

IFACEMETHODIMP
CSiUtilEx::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = nullptr;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_IDispatch))
            *ppvObject = static_cast<IDispatch*>(this);
        else if(IsEqualIID(riid, IID_ISiUtilEx))
            *ppvObject = static_cast<ISiUtilEx*>(this);
        else if(IsEqualIID(riid, IID_IProvideClassInfo))
            *ppvObject = static_cast<IProvideClassInfo*>(this);
        else if(IsEqualIID(riid, IID_ISupportErrorInfo))
            *ppvObject = static_cast<ISupportErrorInfo*>(this);

        if(*ppvObject)
        {
            static_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

IFACEMETHODIMP_(ULONG)
CSiUtilEx::AddRef()
{
    return(InterlockedIncrement(&m_lRefCount));
}

IFACEMETHODIMP_(ULONG)
CSiUtilEx::Release()
{
    ULONG ulResult;

    if(0 == (ulResult = InterlockedDecrement(&m_lRefCount)))
        delete this;

    return(ulResult);
}

IFACEMETHODIMP
CSiUtilEx::GetTypeInfoCount(UINT* pctinfo)
{
    if(IsBadWritePtrT(pctinfo))
        return(E_POINTER);

    *pctinfo = 1;
    return(S_OK);
}

IFACEMETHODIMP
CSiUtilEx::GetTypeInfo(UINT iTInfo, LCID, ITypeInfo** ppTInfo)
{
    if(IsBadWritePtrT(ppTInfo))
        return(E_POINTER);

    if(iTInfo)
    {
        *ppTInfo = nullptr;
        return(DISP_E_BADINDEX);
    }

    *ppTInfo = m_pTypeInfo;
    (*ppTInfo)->AddRef();
    return(S_OK);
}

IFACEMETHODIMP
CSiUtilEx::GetIDsOfNames(REFIID, LPOLESTR* rgszNames, UINT cNames, LCID, DISPID* rgDispId)
{
    return(m_pTypeInfo->GetIDsOfNames(rgszNames, cNames, rgDispId));
}

IFACEMETHODIMP
CSiUtilEx::Invoke(DISPID dispIdMember, REFIID, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr)
{
    CThreadLocale loc(lcid);
    UINT uArgErr;
    return(m_pTypeInfo->Invoke(static_cast<ISiUtilEx*>(this), dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, (puArgErr) ? puArgErr : &uArgErr));
}

IFACEMETHODIMP
CSiUtilEx::get_CommPort(long* plCommPort)
{
    if(IsBadWritePtrT(plCommPort))
        return(RaiseCOMException(DISPID_SIUTILEX_COMMPORT, E_POINTER));

#ifndef _MSC_VER
    *plCommPort = InterlockedCompareExchangeAcquire(&m_lCommPort, -1, -1);
#else
    // Microsoft uses Acquire semantics on volatile variables.
    *plCommPort = m_lCommPort;
#endif

    return(S_OK);
}

IFACEMETHODIMP
CSiUtilEx::put_CommPort(long lCommPort)
{
    InterlockedExchange(&m_lCommPort, lCommPort);
    return(S_OK);
}

IFACEMETHODIMP
CSiUtilEx::get_DisableDialogBoxes(VARIANT_BOOL* pblDisable)
{
    if(IsBadWritePtrT(pblDisable))
        return(RaiseCOMException(DISPID_SIUTILEX_DISABLEDIALOGBOXES, E_POINTER));

#ifndef _MSC_VER
    *pblDisable = (InterlockedCompareExchangeAcquire(&m_nDisableDialogBoxes, -1, -1)) ? VARIANT_TRUE : VARIANT_FALSE;
#else
    // Microsoft uses Acquire semantics on volatile variables.
    *pblDisable = (m_nDisableDialogBoxes) ? VARIANT_TRUE : VARIANT_FALSE;
#endif

    return(S_OK);
}

IFACEMETHODIMP
CSiUtilEx::put_DisableDialogBoxes(VARIANT_BOOL blDisable)
{
    InterlockedExchange(&m_nDisableDialogBoxes, (blDisable != VARIANT_FALSE));
    return(S_OK);
}

IFACEMETHODIMP
CSiUtilEx::get_Version(BSTR* pbstrVersion)
{
    if(IsBadWritePtrT(pbstrVersion))
        return(RaiseCOMException(DISPID_SIUTILEX_VERSION, E_POINTER));

    const HRESULT hr = BSTRFromStringA(::GetDLLVersion(), pbstrVersion);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_VERSION, hr));
}

IFACEMETHODIMP
CSiUtilEx::Connect(enum ECprotocol proto, enum BaudRateIndex baud)
{
#ifndef _MSC_VER
    const HRESULT hr = ::Connect(InterlockedCompareExchangeAcquire(&m_lCommPort, -1, -1), InterlockedCompareExchangeAcquire(&m_nDisableDialogBoxes, -1, -1), static_cast<int>(proto), static_cast<int>(baud));
#else
    // Microsoft uses Acquire semantics on volatile variables.
    const HRESULT hr = ::Connect(m_lCommPort, m_nDisableDialogBoxes, static_cast<int>(proto), static_cast<int>(baud));
#endif

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_CONNECT, hr));
}

IFACEMETHODIMP
CSiUtilEx::Disconnect()
{
#ifndef _MSC_VER
    const HRESULT hr = ::Disconnect(InterlockedCompareExchangeAcquire(&m_lCommPort, -1, -1));
#else
    // Microsoft uses Acquire semantics on volatile variables.
    const HRESULT hr = ::Disconnect(m_lCommPort);
#endif

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_DISCONNECT, hr));
}

IFACEMETHODIMP
CSiUtilEx::Connected(VARIANT_BOOL* pblConnected)
{
    if(IsBadWritePtrT(pblConnected))
        return(RaiseCOMException(DISPID_SIUTILEX_CONNECTED, E_POINTER));

    *pblConnected = (::Connected()) ? VARIANT_TRUE : VARIANT_FALSE;
    return(S_OK);
}

IFACEMETHODIMP
CSiUtilEx::SetTargetGo()
{
    const HRESULT hr = ::SetTargetGo();
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETTARGETGO, hr));
}

IFACEMETHODIMP
CSiUtilEx::SetTargetHalt()
{
    const HRESULT hr = ::SetTargetHalt();
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETTARGETHALT, hr));
}

IFACEMETHODIMP
CSiUtilEx::ISupportBanking(long* pnSupportedBanks)
{
    int nSupportedBanks = 0;
    const HRESULT hr = ::ISupportBanking(&nSupportedBanks);
    *pnSupportedBanks = nSupportedBanks;
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_ISUPPORTBANKING, hr));
}

IFACEMETHODIMP
CSiUtilEx::Download(BSTR sDownloadFile, VARIANT_BOOL blDeviceErase, VARIANT_BOOL blDownloadScratchPadSFLE, long nBankSelect, VARIANT_BOOL blLockFlash, VARIANT_BOOL blPersistFlash, long* usChecksum)
{
#ifndef _MSC_VER
    const HRESULT hr = ::DownloadW(sDownloadFile, blDeviceErase, InterlockedCompareExchangeAcquire(&m_nDisableDialogBoxes, -1, -1), nDownloadScratchPadSFLE, nBankSelect, nLockFlash, bPersistFlash, usChecksum);
#else
    // Microsoft uses Acquire semantics on volatile variables.
    const HRESULT hr = ::DownloadW(sDownloadFile, blDeviceErase, m_nDisableDialogBoxes, blDownloadScratchPadSFLE, nBankSelect, blLockFlash, blPersistFlash, usChecksum);
#endif

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_DOWNLOAD, hr));
}

IFACEMETHODIMP
CSiUtilEx::TargetReset()
{
    const HRESULT hr = ::TargetReset();
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_TARGETRESET, hr));
}

IFACEMETHODIMP
CSiUtilEx::GetRAMMemory(long nStartAddress, long nLength, SAFEARRAY** ppMem)
{
    const HRESULT hr = ::GetMemory(::GetRAMMemory, nStartAddress, nLength, ppMem);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_GETRAMMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilEx::GetXRAMMemory(long nStartAddress, long nLength, SAFEARRAY** ppMem)
{
    const HRESULT hr = ::GetMemory(::GetXRAMMemory, nStartAddress, nLength, ppMem);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_GETXRAMMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilEx::GetCodeMemory(long nStartAddress, long nLength, SAFEARRAY** ppMem)
{
    const HRESULT hr = ::GetMemory(::GetCodeMemory, nStartAddress, nLength, ppMem);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_GETCODEMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilEx::SetRAMMemory(long nStartAddress, SAFEARRAY** ppMem)
{
#ifndef _MSC_VER
    const HRESULT hr = ::SetMemory(::SetRAMMemory, InterlockedCompareExchangeAcquire(&m_nDisableDialogBoxes, -1, -1), nStartAddress, ppMem);
#else
    // Microsoft uses Acquire semantics on volatile variables.
	const HRESULT hr = ::SetMemory(::SetRAMMemory, m_nDisableDialogBoxes, nStartAddress, ppMem);
#endif

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETRAMMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilEx::SetXRAMMemory(long nStartAddress, SAFEARRAY** ppMem)
{
#ifndef _MSC_VER
    const HRESULT hr = ::SetMemory(::SetXRAMMemory, InterlockedCompareExchangeAcquire(&m_nDisableDialogBoxes, -1, -1), nStartAddress, ppMem);
#else
    // Microsoft uses Acquire semantics on volatile variables.
    const HRESULT hr = ::SetMemory(::SetXRAMMemory, m_nDisableDialogBoxes, nStartAddress, ppMem);
#endif
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETXRAMMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilEx::SetCodeMemory(long nStartAddress, SAFEARRAY** ppMem)
{
#ifndef _MSC_VER
#pragma warning(suppress: 4191)
    const HRESULT hr = ::SetMemory((GETSETMEMORYFUNCTION)::SetCodeMemory, InterlockedCompareExchangeAcquire(&m_nDisableDialogBoxes, -1, -1), nStartAddress, ppMem);
#else
    // Microsoft uses Acquire semantics on volatile variables.
#pragma warning(suppress: 4191)
    const HRESULT hr = ::SetMemory(reinterpret_cast<GETSETMEMORYFUNCTION>(::SetCodeMemory), m_nDisableDialogBoxes, nStartAddress, ppMem);
#endif

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETCODEMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilEx::FLASHErase(enum ECprotocol proto)
{
#ifndef _MSC_VER
    const HRESULT hr = ::FLASHErase(InterlockedCompareExchangeAcquire(&m_lCommPort, -1, -1), InterlockedCompareExchangeAcquire(&m_nDisableDialogBoxes, -1, -1), static_cast<int>(proto));
#else
    // Microsoft uses Acquire semantics on volatile variables.
    const HRESULT hr = ::FLASHErase(m_lCommPort, m_nDisableDialogBoxes, static_cast<int>(proto));
#endif

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_FLASHERASE, hr));
}

IFACEMETHODIMP
CSiUtilEx::GetSAFirmwareVersion(long* pnVersion)
{
    if(IsBadWritePtrT(pnVersion))
        return(RaiseCOMException(DISPID_SIUTILEX_GETSAFIRMWAREVERSION, E_POINTER));

    *pnVersion = ::GetSAFirmwareVersion();
    return(S_OK);
}

IFACEMETHODIMP
CSiUtilEx::SetJTAGDeviceAndConnect(long nDevicesBeforeTarget, long nDevicesAfterTarget, long IRBitsBeforeTarget, long IRBitsAfterTarget)
{
    if((static_cast<unsigned long>(nDevicesBeforeTarget) > MAXBYTE) || (static_cast<unsigned long>(nDevicesAfterTarget) > MAXBYTE) || (static_cast<unsigned long>(IRBitsBeforeTarget) > MAXWORD) || (static_cast<unsigned long>(IRBitsAfterTarget) > MAXWORD))
        return(RaiseCOMException(DISPID_SIUTILEX_SETJTAGDEVICEANDCONNECT, E_INVALIDARG));

#ifndef _MSC_VER
    const HRESULT hr = ::SetJTAGDeviceAndConnect(InterlockedCompareExchangeAcquire(&m_lCommPort, -1, -1), InterlockedCompareExchangeAcquire(&m_nDisableDialogBoxes, -1, -1), static_cast<BYTE>(nDevicesBeforeTarget), static_cast<BYTE>(nDevicesAfterTarget), static_cast<WORD>(IRBitsBeforeTarget), static_cast<WORD>(IRBitsAfterTarget));
#else
    // Microsoft uses Acquire semantics on volatile variables.
    const HRESULT hr = ::SetJTAGDeviceAndConnect(m_lCommPort, m_nDisableDialogBoxes, static_cast<BYTE>(nDevicesBeforeTarget), static_cast<BYTE>(nDevicesAfterTarget), static_cast<WORD>(IRBitsBeforeTarget), static_cast<WORD>(IRBitsAfterTarget));
#endif

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETJTAGDEVICEANDCONNECT, hr));
}

IFACEMETHODIMP
CSiUtilEx::GetDeviceName(BSTR* psDeviceName)
{
    if(IsBadWritePtrT(psDeviceName))
        return(RaiseCOMException(DISPID_SIUTILEX_GETDEVICENAME, E_INVALIDARG));

    *psDeviceName = nullptr;
    const char* str;
    HRESULT hr = ::GetDeviceName(&str);

    if(SUCCEEDED(hr))
        hr = BSTRFromStringA(str, psDeviceName);

    return((SUCCEEDED(hr)) ? S_OK : RaiseCOMException(DISPID_SIUTILEX_GETDEVICENAME, hr));
}

IFACEMETHODIMP
CSiUtilEx::SetAppName(BSTR AppName)
{
    const HRESULT hr = ::SetAppNameW(AppName);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETAPPNAME, hr));
}

IFACEMETHODIMP
CSiUtilEx::GetScratchPadMemory(long nStartAddress, long nLength, SAFEARRAY** ppMem)
{
    const HRESULT hr = ::GetMemory(::GetScratchPadMemory, nStartAddress, nLength, ppMem);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_GETSCRATCHPADMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilEx::SetScratchPadMemory(long nStartAddress, SAFEARRAY** ppMem)
{
#ifndef _MSC_VER
#pragma warning(suppress: 4191)
    const HRESULT hr = ::SetMemory(reinterpret_cast<GETSETMEMORYFUNCTION>(::SetScratchPadMemory), InterlockedCompareExchangeAcquire(&m_nDisableDialogBoxes, -1, -1), nStartAddress, ppMem);
#else
    // Microsoft uses Acquire semantics on volatile variables.
#pragma warning(suppress: 4191)
    const HRESULT hr = ::SetMemory(reinterpret_cast<GETSETMEMORYFUNCTION>(::SetScratchPadMemory), m_nDisableDialogBoxes, nStartAddress, ppMem);
#endif

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETSCRATCHPADMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilEx::VerifyFlashDownload(BSTR sDownloadFile, long nBankSelect, VARIANT_BOOL* pblSuccess)
{
#ifndef _MSC_VER
    const HRESULT hr = ::VerifyFlashDownloadW(sDownloadFile, InterlockedCompareExchangeAcquire(&m_nDisableDialogBoxes, -1, -1), nBankSelect, pblSuccess);
#else
    // Microsoft uses Acquire semantics on volatile variables.
    const HRESULT hr = ::VerifyFlashDownloadW(sDownloadFile, m_nDisableDialogBoxes, nBankSelect, pblSuccess);
#endif

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_VERIFYFLASHDOWNLOAD, hr));
}

IFACEMETHODIMP
CSiUtilEx::DownloadTxt(BSTR sDownloadFile, long nStartAddress, VARIANT_BOOL blDeviceErase, VARIANT_BOOL blLockFlash, VARIANT_BOOL blPersistFlash, long* usChecksum)
{
#ifndef _MSC_VER
    const HRESULT hr = ::DownloadTxtW(sDownloadFile, nStartAddress, blDeviceErase, InterlockedCompareExchangeAcquire(&m_nDisableDialogBoxes, -1, -1), blLockFlash, blPersistFlash, usChecksum);
#else
    // Microsoft uses Acquire semantics on volatile variables.
    const HRESULT hr = ::DownloadTxtW(sDownloadFile, nStartAddress, blDeviceErase, m_nDisableDialogBoxes, blLockFlash, blPersistFlash, usChecksum);
#endif

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_DOWNLOADTXT, hr));
}

IFACEMETHODIMP
CSiUtilEx::GetDeviceRevision(long* pPartRev)
{
    if(IsBadWritePtrT(pPartRev))
        return(RaiseCOMException(DISPID_SIUTILEX_GETDEVICEREVISION, E_POINTER));

    BYTE rev = 0;
    const HRESULT hr = ::GetDeviceRevision(&rev);
    *pPartRev = static_cast<long>(static_cast<unsigned long>(rev));
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_GETDEVICEREVISION, hr));
}

IFACEMETHODIMP
CSiUtilEx::GetClassInfo(ITypeInfo** ppTI)
{
    ITypeLib*   pTlb = nullptr;
    HRESULT     hr;
    UINT        uIndex;

    if(IsBadWritePtrT(ppTI))
        return(E_POINTER);

    *ppTI = nullptr;

    if(SUCCEEDED(hr = m_pTypeInfo->GetContainingTypeLib(&pTlb, &uIndex)))
    {
        hr = pTlb->GetTypeInfoOfGuid(CLSID_SiUtilEx, ppTI);
        pTlb->Release();
    }

    return(hr);
}

IFACEMETHODIMP
CSiUtilEx::InterfaceSupportsErrorInfo(REFIID riid)
{
    __try
    {
        if(IsEqualIID(riid, IID_ISiUtilEx))
            return(S_OK);
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(S_FALSE);
}
