#include "precomp.h"

// Map at least some of the stupid HRESULTs returned by SiUtil.dll
// to meaningful values.
static HRESULT MapResult(HRESULT hr)
{
    static const struct
    {
        HRESULT hrSilabs;
        HRESULT hrStandard;
    } sc_Map[] =
    {
        { MAKE_HRESULT(SEVERITY_ERROR, 0x0100, 0x10), E_INVALIDARG                              },
        { MAKE_HRESULT(SEVERITY_ERROR, 0x0100, 0x1E), __HRESULT_FROM_WIN32(ERROR_TIMEOUT)       },
        { MAKE_HRESULT(SEVERITY_ERROR, 0x0100, 0x24), __HRESULT_FROM_WIN32(ERROR_CRC)           },
        { MAKE_HRESULT(SEVERITY_ERROR, 0x0100, 0x26), __HRESULT_FROM_WIN32(ERROR_NOT_SUPPORTED) },
        { MAKE_HRESULT(SEVERITY_ERROR, 0x0104, 0x01), E_HANDLE                                  },
        { MAKE_HRESULT(SEVERITY_ERROR, 0x0104, 0x07), __HRESULT_FROM_WIN32(ERROR_TIMEOUT)       }
    };

    for(uintptr_t idx = 0; idx < _countof(sc_Map); ++idx)
        if(sc_Map[idx].hrSilabs == hr)
            return(sc_Map[idx].hrStandard);

    return(hr);
}

static inline BOOL IsInterfaceSpecificError(HRESULT hr)
{
    // This is because of the HRESULTs returned by SiUtil.dll
    // don't follow any rules.
    return(HRESULT_FACILITY(hr) >= 256);
}

HRESULT RaiseCOMException(IDispatch* pDispatch, DISPID dispid, HRESULT hr)
{
    IProvideClassInfo*  pPCI        = nullptr;
    ICreateErrorInfo*   pCreateErr  = nullptr;
    IErrorInfo*         pError      = nullptr;
    ITypeInfo*          pInfo       = nullptr;
    TYPEATTR*           ptattr      = nullptr;
    LPOLESTR            pSource     = nullptr;
    DWORD               dwHelpContext;
    TCHAR               szError[1024];
    BSTR                bstrHelpFile = nullptr;
    bool                blHasMessage;

    if(IsInterfaceSpecificError(hr))
    {
        #ifdef UNICODE
          MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED | MB_ERR_INVALID_CHARS, ::GetErrorMsg(hr), -1, szError, _countof(szError));
        #else
          StringCchCopyA(szError, _countof(szError), ::GetErrorMsg(hr));
        #endif

        szError[_countof(szError) - 1] = TEXT('\0');
        blHasMessage = (szError[0] != TEXT('\0'));
    }
    else
    {
        blHasMessage = (0 != FormatMessage(FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_SYSTEM, nullptr, static_cast<DWORD>(hr), 0, szError, _countof(szError), nullptr));
    }

    if(!blHasMessage)
        StringCchPrintf(szError, _countof(szError), TEXT("Unknown error: 0x%08lX"), hr);

    if(SUCCEEDED(CreateErrorInfo(&pCreateErr)))
    {
        #ifdef UNICODE
          pCreateErr->SetDescription(szError);
        #else
          WCHAR wszError[_countof(szError)];
          MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED | MB_ERR_INVALID_CHARS, szError, -1, wszError, _countof(wszError));
          wszError[_countof(szError) - 1] = TEXT('\0');
          pCreateErr->SetDescription(wszError);
        #endif

        if(SUCCEEDED(pDispatch->QueryInterface(IID_IProvideClassInfo, reinterpret_cast<void**>(&pPCI))))
        {
            if(SUCCEEDED(pPCI->GetClassInfo(&pInfo)))
            {
                if(SUCCEEDED(pInfo->GetTypeAttr(&ptattr)))
                {
                    if(ptattr->wTypeFlags & TYPEFLAG_FCANCREATE)
                    {
                        if(SUCCEEDED(ProgIDFromCLSID(ptattr->guid, &pSource)))
                        {
                            pCreateErr->SetSource(pSource);
                            CoTaskMemFree(pSource);
                        }
                    }
                    pInfo->ReleaseTypeAttr(ptattr);
                    ptattr = nullptr;
                }
                pInfo->Release();
                pInfo = nullptr;
            }
            pPCI->Release();
        }

        if(SUCCEEDED(pDispatch->GetTypeInfo(0, CThreadLocale::getLCID(), &pInfo)))
        {
            if(SUCCEEDED(pInfo->GetTypeAttr(&ptattr)))
            {
                pCreateErr->SetGUID(ptattr->guid);
                pInfo->ReleaseTypeAttr(ptattr);
            }

            if(SUCCEEDED(pInfo->GetDocumentation(dispid, nullptr, nullptr, &dwHelpContext, &bstrHelpFile)))
            {
                pCreateErr->SetHelpContext(dwHelpContext);
                pCreateErr->SetHelpFile(bstrHelpFile);
                SysFreeString(bstrHelpFile);
            }

            pInfo->Release();
        }

        if(SUCCEEDED(pCreateErr->QueryInterface(IID_IErrorInfo, reinterpret_cast<void**>(&pError))))
        {
            SetErrorInfo(0, pError);
            pError->Release();
        }

        pCreateErr->Release();
    }

    return(MapResult(hr));
}
